@@ -12,9 +12,7 @@
 # limitations under the License.
 
 import os
-import subprocess
 import sys
-import warnings
 
 on_rtd = os.environ.get('READTHEDOCS', None) == 'True'
 
@@ -30,18 +28,18 @@ sys.path.insert(0, os.path.abspath('./'))
 # extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
 extensions = [
     'sphinx.ext.autodoc',
-    'sphinxcontrib.autohttp.flask',
     'sphinxcontrib.pecanwsme.rest',
+    'sphinxcontrib.httpdomain',
     'wsmeext.sphinxext',
+    'openstackdocstheme',
 ]
 
-if not on_rtd:
-    extensions.append('oslosphinx')
-
 wsme_protocols = ['restjson']
 
+suppress_warnings = ['app.add_directive']
+
 # Add any paths that contain templates here, relative to this directory.
-templates_path = ['_templates']
+# templates_path = ['_templates']
 
 # autodoc generation is a bit aggressive and a nuisance when doing heavy
 # text edit cycles.
@@ -84,9 +82,7 @@ pygments_style = 'sphinx'
 # a list of builtin themes.
 # html_static_path = ['_static']
 
-if on_rtd:
-    html_theme_path = ['.']
-    html_theme = 'sphinx_rtd_theme'
+html_theme = 'openstackdocs'
 
 # Output file base name for HTML help builder.
 htmlhelp_basename = '%sdoc' % project
@@ -96,15 +92,9 @@ modindex_common_prefix = ['mistral.']
 
 # If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
 # using the given strftime format.
-# html_last_updated_fmt = '%b %d, %Y'
-git_cmd = ["git", "log", "--pretty=format:'%ad, commit %h'", "--date=local",
-           "-n1"]
-try:
-    html_last_updated_fmt = subprocess.Popen(
-        git_cmd, stdout=subprocess.PIPE).communicate()[0]
-except Exception:
-    warnings.warn('Cannot get last updated time from git repository. '
-                  'Not setting "html_last_updated_fmt".')
+#html_last_updated_fmt = '%b %d, %Y'
+# Must set this variable to include year, month, day, hours, and minutes.
+html_last_updated_fmt = '%Y-%m-%d %H:%M'
 
 # The name for this set of Sphinx documents. If None, it defaults to
 # "<project> v<release> documentation".
@@ -133,3 +123,8 @@ man_pages = [
 
 # If true, show URL addresses after external links.
 man_show_urls = True
+
+# -- Options for openstackdocstheme -------------------------------------------
+repository_name = 'openstack/mistral'
+bug_project = 'mistral'
+bug_tag = ''
