@@ -26,8 +26,8 @@ from mistral.api.controllers.v2 import task
 from mistral.api.controllers.v2 import types
 from mistral import context
 from mistral.db.v2 import api as db_api
-from mistral.engine.rpc_backend import rpc
 from mistral import exceptions as exc
+from mistral.rpc import clients as rpc
 from mistral.services import workflows as wf_service
 from mistral.utils import filter_utils
 from mistral.utils import rest_utils
@@ -47,12 +47,12 @@ STATE_TYPES = wtypes.Enum(
 )
 
 
-def _get_execution_resource(ex):
+def _get_execution_resource(wf_ex):
     # We need to refer to this lazy-load field explicitly in
     # order to make sure that it is correctly loaded.
-    hasattr(ex, 'output')
+    hasattr(wf_ex, 'output')
 
-    return resources.Execution.from_dict(ex.to_dict())
+    return resources.Execution.from_db_model(wf_ex)
 
 
 # TODO(rakhmerov): Make sure to make all needed renaming on public API.
@@ -64,7 +64,10 @@ class ExecutionsController(rest.RestController):
     @rest_utils.wrap_wsme_controller_exception
     @wsme_pecan.wsexpose(resources.Execution, wtypes.text)
     def get(self, id):
-        """Return the specified Execution."""
+        """Return the specified Execution.
+
+        :param id: UUID of execution to retrieve.
+        """
         acl.enforce("executions:get", context.ctx())
 
         LOG.info("Fetch execution [id=%s]", id)
@@ -77,7 +80,7 @@ class ExecutionsController(rest.RestController):
             # amount of DB queries and network traffic.
             hasattr(wf_ex, 'output')
 
-        return resources.Execution.from_dict(wf_ex.to_dict())
+        return resources.Execution.from_db_model(wf_ex)
 
     @rest_utils.wrap_wsme_controller_exception
     @wsme_pecan.wsexpose(
@@ -88,12 +91,12 @@ class ExecutionsController(rest.RestController):
     def put(self, id, wf_ex):
         """Update the specified workflow execution.
 
-        :param id: execution ID.
+        :param id: UUID of execution to update.
         :param wf_ex: Execution object.
         """
         acl.enforce('executions:update', context.ctx())
 
-        LOG.info('Update execution [id=%s, execution=%s]' % (id, wf_ex))
+        LOG.info('Update execution [id=%s, execution=%s]', id, wf_ex)
 
         with db_api.transaction():
             db_api.ensure_workflow_execution_exists(id)
@@ -205,6 +208,7 @@ class ExecutionsController(rest.RestController):
 
         result = engine.start_workflow(
             exec_dict.get('workflow_id', exec_dict.get('workflow_name')),
+            exec_dict.get('workflow_namespace', ''),
             exec_dict.get('input'),
             exec_dict.get('description', ''),
             **exec_dict.get('params') or {}
@@ -215,7 +219,10 @@ class ExecutionsController(rest.RestController):
     @rest_utils.wrap_wsme_controller_exception
     @wsme_pecan.wsexpose(None, wtypes.text, status_code=204)
     def delete(self, id):
-        """Delete the specified Execution."""
+        """Delete the specified Execution.
+
+        :param id: UUID of execution to delete.
+        """
         acl.enforce('executions:delete', context.ctx())
 
         LOG.info("Delete execution [id=%s]", id)
@@ -227,13 +234,14 @@ class ExecutionsController(rest.RestController):
                          types.uniquelist, types.list, types.uniquelist,
                          wtypes.text, types.uuid, wtypes.text, types.jsontype,
                          types.uuid, STATE_TYPES, wtypes.text, types.jsontype,
-                         types.jsontype, wtypes.text, wtypes.text, bool)
+                         types.jsontype, wtypes.text, wtypes.text, bool,
+                         types.uuid, bool)
     def get_all(self, marker=None, limit=None, sort_keys='created_at',
                 sort_dirs='asc', fields='', workflow_name=None,
                 workflow_id=None, description=None, params=None,
                 task_execution_id=None, state=None, state_info=None,
                 input=None, output=None, created_at=None, updated_at=None,
-                include_output=None):
+                include_output=None, project_id=None, all_projects=False):
         """Return all Executions.
 
         :param marker: Optional. Pagination marker for large data sets.
@@ -269,10 +277,17 @@ class ExecutionsController(rest.RestController):
         :param updated_at: Optional. Keep only resources with specific latest
                            update time and date.
         :param include_output: Optional. Include the output for all executions
-                               in the list
+                               in the list.
+        :param project_id: Optional. Only get exectuions belong to the project.
+            Admin required.
+        :param all_projects: Optional. Get resources of all projects. Admin
+            required.
         """
         acl.enforce('executions:list', context.ctx())
 
+        if all_projects or project_id:
+            acl.enforce('executions:list:all_projects', context.ctx())
+
         filters = filter_utils.create_filters_from_request_params(
             created_at=created_at,
             workflow_name=workflow_name,
@@ -284,13 +299,14 @@ class ExecutionsController(rest.RestController):
             input=input,
             output=output,
             updated_at=updated_at,
-            description=description
+            description=description,
+            project_id=project_id
         )
 
         LOG.info(
             "Fetch executions. marker=%s, limit=%s, sort_keys=%s, "
-            "sort_dirs=%s, filters=%s", marker, limit, sort_keys, sort_dirs,
-            filters
+            "sort_dirs=%s, filters=%s, all_projects=%s", marker, limit,
+            sort_keys, sort_dirs, filters, all_projects
         )
 
         if include_output:
@@ -309,5 +325,6 @@ class ExecutionsController(rest.RestController):
             sort_keys=sort_keys,
             sort_dirs=sort_dirs,
             fields=fields,
+            all_projects=all_projects,
             **filters
         )
