@@ -44,9 +44,9 @@ from oslo_service import service
 from mistral.api import service as api_service
 from mistral import config
 from mistral.engine import engine_server
-from mistral.engine import executor_server
-from mistral.engine.rpc_backend import rpc
 from mistral.event_engine import event_engine_server
+from mistral.executors import executor_server
+from mistral.rpc import base as rpc
 from mistral import version
 
 
@@ -118,13 +118,12 @@ LAUNCH_OPTIONS = {
 
 
 MISTRAL_TITLE = """
-|\\\    //| //   // |||||| |||\\\       /\      ||
-||\\\  //||    //     ||   ||  ||     //\\\     ||
-|| \\\// || || ||     ||   || //     //  \\\    ||
-||  \/  || ||  \\\    ||   || \\\    //-||-\\\   ||
-||      || ||   ||   ||   ||  ||  //      \\\  ||
-||      || || _//    ||   ||  || //        \\\ |||||
-
+|\\\    //|           ||                       ||
+||\\\  //||      __   ||      __      __       ||
+|| \\\// || ||  //  ||||||  ||  \\\  //  \\\     ||
+||  \\/  ||     \\\    ||    ||     ||    \\\    ||
+||      || ||   \\\   ||    ||     ||    /\\\   ||
+||      || || __//   ||_// ||      \\\__// \\\_ ||
 Mistral Workflow Service, version %s
 """ % version.version_string()
 
