@@ -17,7 +17,7 @@ from oslo_log import log as logging
 from mistral import config as cfg
 from mistral.db.v2 import api as db_api
 from mistral.engine import default_engine
-from mistral.engine.rpc_backend import rpc
+from mistral.rpc import base as rpc
 from mistral.service import base as service_base
 from mistral.services import expiration_policy
 from mistral.services import scheduler
@@ -78,12 +78,13 @@ class EngineServer(service_base.MistralService):
         if self._rpc_server:
             self._rpc_server.stop(graceful)
 
-    def start_workflow(self, rpc_ctx, workflow_identifier, workflow_input,
-                       description, params):
+    def start_workflow(self, rpc_ctx, workflow_identifier, workflow_namespace,
+                       workflow_input, description, params):
         """Receives calls over RPC to start workflows on engine.
 
         :param rpc_ctx: RPC request context.
         :param workflow_identifier: Workflow definition identifier.
+        :param workflow_namespace: Workflow definition identifier.
         :param workflow_input: Workflow input.
         :param description: Workflow execution description.
         :param params: Additional workflow type specific parameters.
@@ -91,15 +92,17 @@ class EngineServer(service_base.MistralService):
         """
 
         LOG.info(
-            "Received RPC request 'start_workflow'[rpc_ctx=%s,"
-            " workflow_identifier=%s, workflow_input=%s, description=%s, "
-            "params=%s]" %
-            (rpc_ctx, workflow_identifier, utils.cut(workflow_input),
-             description, params)
+            "Received RPC request 'start_workflow'[workflow_identifier=%s, "
+            "workflow_input=%s, description=%s, params=%s]",
+            workflow_identifier,
+            utils.cut(workflow_input),
+            description,
+            params
         )
 
         return self.engine.start_workflow(
             workflow_identifier,
+            workflow_namespace,
             workflow_input,
             description,
             **params
@@ -117,10 +120,12 @@ class EngineServer(service_base.MistralService):
         :return: Action execution.
         """
         LOG.info(
-            "Received RPC request 'start_action'[rpc_ctx=%s,"
-            " name=%s, input=%s, description=%s, params=%s]"
-            % (rpc_ctx, action_name, utils.cut(action_input),
-               description, params)
+            "Received RPC request 'start_action'[name=%s, input=%s, "
+            "description=%s, params=%s]",
+            action_name,
+            utils.cut(action_input),
+            description,
+            params
         )
 
         return self.engine.start_action(
@@ -139,15 +144,33 @@ class EngineServer(service_base.MistralService):
         :param wf_action: True if given id points to a workflow execution.
         :return: Action execution.
         """
-
         LOG.info(
-            "Received RPC request 'on_action_complete'[rpc_ctx=%s,"
-            " action_ex_id=%s, result=%s]" %
-            (rpc_ctx, action_ex_id, result.cut_repr())
+            "Received RPC request 'on_action_complete'[action_ex_id=%s, "
+            "result=%s]",
+            action_ex_id,
+            result.cut_repr()
         )
 
         return self.engine.on_action_complete(action_ex_id, result, wf_action)
 
+    def on_action_update(self, rpc_ctx, action_ex_id, state, wf_action):
+        """Receives RPC calls to communicate action execution state to engine.
+
+        :param rpc_ctx: RPC request context.
+        :param action_ex_id: Action execution id.
+        :param state: Action execution state.
+        :param wf_action: True if given id points to a workflow execution.
+        :return: Action execution.
+        """
+        LOG.info(
+            "Received RPC request 'on_action_update'"
+            "[action_ex_id=%s, state=%s]",
+            action_ex_id,
+            state
+        )
+
+        return self.engine.on_action_update(action_ex_id, state, wf_action)
+
     def pause_workflow(self, rpc_ctx, execution_id):
         """Receives calls over RPC to pause workflows on engine.
 
@@ -155,10 +178,9 @@ class EngineServer(service_base.MistralService):
         :param execution_id: Workflow execution id.
         :return: Workflow execution.
         """
-
         LOG.info(
-            "Received RPC request 'pause_workflow'[rpc_ctx=%s,"
-            " execution_id=%s]" % (rpc_ctx, execution_id)
+            "Received RPC request 'pause_workflow'[execution_id=%s]",
+            execution_id
         )
 
         return self.engine.pause_workflow(execution_id)
@@ -172,10 +194,9 @@ class EngineServer(service_base.MistralService):
         :param env: Environment variables to update.
         :return: Workflow execution.
         """
-
         LOG.info(
-            "Received RPC request 'rerun_workflow'[rpc_ctx=%s, "
-            "task_ex_id=%s]" % (rpc_ctx, task_ex_id)
+            "Received RPC request 'rerun_workflow'[task_ex_id=%s]",
+            task_ex_id
         )
 
         return self.engine.rerun_workflow(task_ex_id, reset, env)
@@ -188,10 +209,9 @@ class EngineServer(service_base.MistralService):
         :param env: Environment variables to update.
         :return: Workflow execution.
         """
-
         LOG.info(
-            "Received RPC request 'resume_workflow'[rpc_ctx=%s, "
-            "wf_ex_id=%s]" % (rpc_ctx, wf_ex_id)
+            "Received RPC request 'resume_workflow'[wf_ex_id=%s]",
+            wf_ex_id
         )
 
         return self.engine.resume_workflow(wf_ex_id, env)
@@ -211,11 +231,12 @@ class EngineServer(service_base.MistralService):
 
         :return: Workflow execution.
         """
-
         LOG.info(
-            "Received RPC request 'stop_workflow'[rpc_ctx=%s, execution_id=%s,"
-            " state=%s, message=%s]" %
-            (rpc_ctx, execution_id, state, message)
+            "Received RPC request 'stop_workflow'[execution_id=%s,"
+            " state=%s, message=%s]",
+            execution_id,
+            state,
+            message
         )
 
         return self.engine.stop_workflow(execution_id, state, message)
@@ -226,10 +247,9 @@ class EngineServer(service_base.MistralService):
         :param rpc_ctx: RPC request context.
         :return: Workflow execution.
         """
-
         LOG.info(
-            "Received RPC request 'rollback_workflow'[rpc_ctx=%s,"
-            " execution_id=%s]" % (rpc_ctx, execution_id)
+            "Received RPC request 'rollback_workflow'[execution_id=%s]",
+            execution_id
         )
 
         return self.engine.rollback_workflow(execution_id)
