@@ -35,8 +35,6 @@ INLINE_YAQL_REGEXP = '<%.*?%>'
 class YAQLEvaluator(Evaluator):
     @classmethod
     def validate(cls, expression):
-        LOG.debug("Validating YAQL expression [expression='%s']", expression)
-
         try:
             YAQL_ENGINE(expression)
         except (yaql_exc.YaqlException, KeyError, ValueError, TypeError) as e:
@@ -46,11 +44,6 @@ class YAQLEvaluator(Evaluator):
     def evaluate(cls, expression, data_context):
         expression = expression.strip() if expression else expression
 
-        LOG.debug(
-            "Evaluating YAQL expression [expression='%s', context=%s]"
-            % (expression, data_context)
-        )
-
         try:
             result = YAQL_ENGINE(expression).evaluate(
                 context=expression_utils.get_yaql_context(data_context)
@@ -61,8 +54,6 @@ class YAQLEvaluator(Evaluator):
                 ", data=%s]" % (expression, str(e), data_context)
             )
 
-        LOG.debug("YAQL expression result: %s" % result)
-
         return result if not inspect.isgenerator(result) else list(result)
 
     @classmethod
@@ -79,9 +70,6 @@ class InlineYAQLEvaluator(YAQLEvaluator):
 
     @classmethod
     def validate(cls, expression):
-        LOG.debug(
-            "Validating inline YAQL expression [expression='%s']", expression)
-
         if not isinstance(expression, six.string_types):
             raise exc.YaqlEvaluationException(
                 "Unsupported type '%s'." % type(expression)
@@ -96,8 +84,10 @@ class InlineYAQLEvaluator(YAQLEvaluator):
     @classmethod
     def evaluate(cls, expression, data_context):
         LOG.debug(
-            "Evaluating inline YAQL expression [expression='%s', context=%s]"
-            % (expression, data_context)
+            "Start to evaluate YAQL expression. "
+            "[expression='%s', context=%s]",
+            expression,
+            data_context
         )
 
         result = expression
@@ -113,7 +103,11 @@ class InlineYAQLEvaluator(YAQLEvaluator):
                 else:
                     result = result.replace(expr, str(evaluated))
 
-        LOG.debug("Inline YAQL expression result: %s" % result)
+        LOG.debug(
+            "Finished evaluation. [expression='%s', result: %s]",
+            expression,
+            result
+        )
 
         return result
 
