@@ -1,5 +1,6 @@
 # Copyright 2014 - Mirantis, Inc.
 # Copyright 2015 - StackStorm, Inc.
+# Copyright 2016 - Brocade Communications Systems, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -36,24 +37,33 @@ _schedulers = {}
 
 
 def schedule_call(factory_method_path, target_method_name,
-                  run_after, serializers=None, **method_args):
-    """Add this call specification to DB, and then after run_after
+                  run_after, serializers=None, key=None, **method_args):
+    """Schedules call and lately invokes target_method.
+
+    Add this call specification to DB, and then after run_after
     seconds service CallScheduler invokes the target_method.
 
     :param factory_method_path: Full python-specific path to
-    factory method for target object construction.
-    :param target_method_name: Name of target object method which
-    will be invoked.
+        factory method that creates a target object that the call will be
+        made against.
+    :param target_method_name: Name of a method which will be invoked.
     :param run_after: Value in seconds.
-    param serializers: map of argument names and their serializer class paths.
-     Use when an argument is an object of specific type, and needs to be
-      serialized. Example:
-      { "result": "mistral.utils.serializer.ResultSerializer"}
-      Serializer for the object type must implement serializer interface
-       in mistral/utils/serializer.py
+    :param serializers: map of argument names and their serializer class
+        paths. Use when an argument is an object of specific type, and needs
+        to be serialized. Example:
+        { "result": "mistral.utils.serializer.ResultSerializer"}
+        Serializer for the object type must implement serializer interface
+        in mistral/utils/serializer.py
+    :param key: Key which can potentially be used for squashing similar
+        delayed calls.
     :param method_args: Target method keyword arguments.
     """
-    ctx = context.ctx().to_dict() if context.has_ctx() else {}
+    ctx_serializer = context.RpcContextSerializer()
+
+    ctx = (
+        ctx_serializer.serialize_context(context.ctx())
+        if context.has_ctx() else {}
+    )
 
     execution_time = (datetime.datetime.now() +
                       datetime.timedelta(seconds=run_after))
@@ -68,12 +78,11 @@ def schedule_call(factory_method_path, target_method_name,
             try:
                 serializer = importutils.import_class(serializer_path)()
             except ImportError as e:
-                raise ImportError("Cannot import class %s: %s"
-                                  % (serializer_path, e))
+                raise ImportError(
+                    "Cannot import class %s: %s" % (serializer_path, e)
+                )
 
-            method_args[arg_name] = serializer.serialize(
-                method_args[arg_name]
-            )
+            method_args[arg_name] = serializer.serialize(method_args[arg_name])
 
     values = {
         'factory_method_path': factory_method_path,
@@ -81,6 +90,7 @@ def schedule_call(factory_method_path, target_method_name,
         'execution_time': execution_time,
         'auth_context': ctx,
         'serializers': serializers,
+        'key': key,
         'method_arguments': method_args,
         'processing': False
     }
@@ -95,15 +105,11 @@ class CallScheduler(periodic_task.PeriodicTasks):
         time_filter = datetime.datetime.now() + datetime.timedelta(
             seconds=1)
 
-        # Wrap delayed calls processing in transaction to
-        # guarantee that calls will be processed just once.
-        # Do delete query to DB first to force hanging up all
-        # parallel transactions.
-        # It should work on isolation level 'READ-COMMITTED',
-        # 'REPEATABLE-READ' and above.
-        #
-        # 'REPEATABLE-READ' is by default in MySQL and
-        # 'READ-COMMITTED is by default in PostgreSQL.
+        # Wrap delayed calls processing in transaction to guarantee that calls
+        # will be processed just once. Do delete query to DB first to force
+        # hanging up all parallel transactions.
+        # It should work with transactions which run at least 'READ-COMMITTED'
+        # mode.
         delayed_calls = []
 
         with db_api.transaction():
@@ -118,7 +124,7 @@ class CallScheduler(periodic_task.PeriodicTasks):
                 result, number_of_updated = db_api.update_delayed_call(
                     id=call.id,
                     values={'processing': True},
-                    query_filter={"processing": False}
+                    query_filter={'processing': False}
                 )
 
                 # If number_of_updated != 1 other scheduler already
@@ -127,9 +133,14 @@ class CallScheduler(periodic_task.PeriodicTasks):
                     calls_to_make.append(result)
 
         for call in calls_to_make:
-            LOG.debug('Processing next delayed call: %s', call)
+            LOG.debug(
+                'Processing next delayed call. '
+                '[ID=%s, factory_method_path=%s, target_method_name=%s, '
+                'method_arguments=%s]', call.id, call.factory_method_path,
+                call.target_method_name, call.method_arguments
+            )
 
-            context.set_ctx(context.MistralContext(call.auth_context))
+            target_auth_context = copy.deepcopy(call.auth_context)
 
             if call.factory_method_path:
                 factory = importutils.import_class(
@@ -155,19 +166,28 @@ class CallScheduler(periodic_task.PeriodicTasks):
 
                     method_args[arg_name] = deserialized
 
-            delayed_calls.append((target_method, method_args))
+            delayed_calls.append(
+                (target_auth_context, target_method, method_args)
+            )
 
-        for (target_method, method_args) in delayed_calls:
-            # Transaction is needed here because some of the
-            # target_method can use the DB
-            with db_api.transaction():
-                try:
-                    # Call the method.
-                    target_method(**method_args)
-                except Exception as e:
-                    LOG.error(
-                        "Delayed call failed [exception=%s]", e
-                    )
+        ctx_serializer = context.RpcContextSerializer()
+
+        for (target_auth_context, target_method, method_args) in delayed_calls:
+            try:
+                # Set the correct context for the method.
+                ctx_serializer.deserialize_context(target_auth_context)
+
+                # Call the method.
+                target_method(**method_args)
+            except Exception as e:
+                LOG.exception(
+                    "Delayed call failed, method: %s, exception: %s",
+                    target_method,
+                    e
+                )
+            finally:
+                # Remove context.
+                context.set_ctx(None)
 
         with db_api.transaction():
             for call in calls_to_make:
@@ -181,21 +201,33 @@ class CallScheduler(periodic_task.PeriodicTasks):
                     )
 
 
-def setup():
+def start():
     tg = threadgroup.ThreadGroup()
 
-    scheduler = CallScheduler(CONF)
+    sched = CallScheduler(CONF)
 
     tg.add_dynamic_timer(
-        scheduler.run_periodic_tasks,
+        sched.run_periodic_tasks,
         initial_delay=None,
         periodic_interval_max=1,
         context=None
     )
 
-    _schedulers[scheduler] = tg
+    _schedulers[sched] = tg
 
-    return tg
+    return sched
+
+
+def stop_scheduler(sched, graceful=False):
+    if sched:
+        tg = _schedulers[sched]
+
+        tg.stop()
+
+        del _schedulers[sched]
+
+        if graceful:
+            tg.wait()
 
 
 def stop_all_schedulers():
