@@ -19,7 +19,9 @@ import mock
 from mistral.db.v2 import api as db_api
 from mistral.db.v2.sqlalchemy import models
 from mistral import exceptions as exc
+from mistral.services import security
 from mistral.tests.unit.api import base
+from mistral.tests.unit import base as unit_base
 
 WF = models.WorkflowDefinition(
     spec={
@@ -59,7 +61,7 @@ TRIGGER_DB.update(trigger_values)
 MOCK_WF = mock.MagicMock(return_value=WF)
 MOCK_TRIGGER = mock.MagicMock(return_value=TRIGGER_DB)
 MOCK_TRIGGERS = mock.MagicMock(return_value=[TRIGGER_DB])
-MOCK_DELETE = mock.MagicMock(return_value=None)
+MOCK_DELETE = mock.MagicMock(return_value=1)
 MOCK_EMPTY = mock.MagicMock(return_value=[])
 MOCK_NOT_FOUND = mock.MagicMock(side_effect=exc.DBEntityNotFoundError())
 MOCK_DUPLICATE = mock.MagicMock(side_effect=exc.DBDuplicateEntryError())
@@ -101,11 +103,13 @@ class TestCronTriggerController(base.APITest):
 
     @mock.patch.object(db_api, "get_workflow_definition", MOCK_WF)
     @mock.patch.object(db_api, "create_cron_trigger", MOCK_DUPLICATE)
-    def test_post_dup(self):
+    @mock.patch.object(security, "delete_trust")
+    def test_post_dup(self, delete_trust):
         resp = self.app.post_json(
             '/v2/cron_triggers', TRIGGER, expect_errors=True
         )
 
+        self.assertEqual(1, delete_trust.call_count)
         self.assertEqual(409, resp.status_int)
 
     @mock.patch.object(db_api, "get_workflow_definition", MOCK_WF)
@@ -122,9 +126,11 @@ class TestCronTriggerController(base.APITest):
 
     @mock.patch.object(db_api, "get_cron_trigger", MOCK_TRIGGER)
     @mock.patch.object(db_api, "delete_cron_trigger", MOCK_DELETE)
-    def test_delete(self):
+    @mock.patch.object(security, "delete_trust")
+    def test_delete(self, delete_trust):
         resp = self.app.delete('/v2/cron_triggers/my_cron_trigger')
 
+        self.assertEqual(1, delete_trust.call_count)
         self.assertEqual(204, resp.status_int)
 
     @mock.patch.object(db_api, "delete_cron_trigger", MOCK_NOT_FOUND)
@@ -145,6 +151,18 @@ class TestCronTriggerController(base.APITest):
         self.assertEqual(1, len(resp.json['cron_triggers']))
         self.assertDictEqual(TRIGGER, resp.json['cron_triggers'][0])
 
+    @mock.patch.object(db_api, 'get_cron_triggers')
+    @mock.patch('mistral.context.MistralContext.from_environ')
+    def test_get_all_projects_admin(self, mock_context, mock_get_triggers):
+        admin_ctx = unit_base.get_context(admin=True)
+        mock_context.return_value = admin_ctx
+
+        resp = self.app.get('/v2/cron_triggers?all_projects=true')
+
+        self.assertEqual(200, resp.status_int)
+
+        self.assertTrue(mock_get_triggers.call_args[1].get('insecure', False))
+
     @mock.patch.object(db_api, "get_cron_triggers", MOCK_EMPTY)
     def test_get_all_empty(self):
         resp = self.app.get('/v2/cron_triggers')
