@@ -97,11 +97,11 @@ MOCK_WORKBOOKS = mock.MagicMock(return_value=[WORKBOOK_DB])
 MOCK_UPDATED_WORKBOOK = mock.MagicMock(return_value=UPDATED_WORKBOOK_DB)
 MOCK_DELETE = mock.MagicMock(return_value=None)
 MOCK_EMPTY = mock.MagicMock(return_value=[])
-MOCK_NOT_FOUND = mock.MagicMock(side_effect=exc.NotFoundException())
-MOCK_DUPLICATE = mock.MagicMock(side_effect=exc.DBDuplicateEntryException())
+MOCK_NOT_FOUND = mock.MagicMock(side_effect=exc.DBEntityNotFoundError())
+MOCK_DUPLICATE = mock.MagicMock(side_effect=exc.DBDuplicateEntryError())
 
 
-class TestWorkbooksController(base.FunctionalTest):
+class TestWorkbooksController(base.APITest):
     @mock.patch.object(db_api, "get_workbook", MOCK_WORKBOOK)
     def test_get(self):
         resp = self.app.get('/v2/workbooks/123')
@@ -146,7 +146,7 @@ class TestWorkbooksController(base.FunctionalTest):
         )
 
         self.assertEqual(400, resp.status_int)
-        self.assertIn("Invalid DSL", resp.body)
+        self.assertIn("Invalid DSL", resp.body.decode())
 
     @mock.patch.object(workbooks, "create_workbook_v2", MOCK_WORKBOOK)
     def test_post(self):
@@ -179,7 +179,7 @@ class TestWorkbooksController(base.FunctionalTest):
         )
 
         self.assertEqual(400, resp.status_int)
-        self.assertIn("Invalid DSL", resp.body)
+        self.assertIn("Invalid DSL", resp.body.decode())
 
     @mock.patch.object(db_api, "delete_workbook", MOCK_DELETE)
     def test_delete(self):
