@@ -12,12 +12,12 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 import datetime
-import time
 
 import mock
 from oslo_config import cfg
 
 from mistral.db.v2 import api as db_api
+from mistral import exceptions as exc
 from mistral.services import periodic
 from mistral.services import security
 from mistral.services import triggers
@@ -43,7 +43,9 @@ class ProcessCronTriggerTest(base.EngineTestCase):
     @mock.patch.object(security,
                        'create_trust',
                        type('trust', (object,), {'id': 'my_trust_id'}))
-    def test_start_workflow(self):
+    @mock.patch('mistral.rpc.clients.get_engine_client')
+    def test_start_workflow(self, rpc_mock):
+
         cfg.CONF.set_default('auth_enable', True, group='pecan')
 
         wf = workflows.create_workflows(WORKFLOW_LIST)[0]
@@ -68,6 +70,13 @@ class ProcessCronTriggerTest(base.EngineTestCase):
 
         periodic.MistralPeriodicTasks(cfg.CONF).process_cron_triggers_v2(None)
 
+        start_workflow_mock = rpc_mock.return_value.start_workflow
+        start_workflow_mock.assert_called_once()
+        self.assertIn(
+            t.id,
+            start_workflow_mock.mock_calls[0][2]['description']
+        )
+
         next_triggers = triggers.get_next_cron_triggers()
 
         self.assertEqual(1, len(next_triggers))
@@ -128,7 +137,7 @@ class ProcessCronTriggerTest(base.EngineTestCase):
 
         # Make the first_time 1 sec later than current time, in order to make
         # it executed by next cron-trigger task.
-        first_time = datetime.datetime.now() + datetime.timedelta(0, 1)
+        first_time = datetime.datetime.utcnow() + datetime.timedelta(0, 1)
 
         # Creates a cron-trigger with pattern and first time, ensure the
         # cron-trigger can be executed more than once, and cron-trigger will
@@ -146,11 +155,8 @@ class ProcessCronTriggerTest(base.EngineTestCase):
             None
         )
 
-        first_second = time.mktime(first_time.timetuple())
-        first_utc_time = datetime.datetime.utcfromtimestamp(first_second)
-
         self.assertEqual(
-            first_utc_time,
+            first_time,
             cron_trigger.next_execution_time
         )
 
@@ -168,3 +174,12 @@ class ProcessCronTriggerTest(base.EngineTestCase):
             next_time,
             cron_trigger_db.next_execution_time
         )
+
+    def test_validate_cron_trigger_input_first_time(self):
+        cfg.CONF.set_default('auth_enable', False, group='pecan')
+        first_time = datetime.datetime.utcnow() + datetime.timedelta(0, 1)
+        self.assertRaises(exc.InvalidModelException,
+                          triggers.validate_cron_trigger_input,
+                          None,
+                          first_time,
+                          None)
