@@ -20,7 +20,7 @@ from mistral import exceptions as exc
 from mistral.services import workflows as wf_service
 from mistral.tests.unit.engine import base
 from mistral.workflow import states
-from mistral.workflow import utils as wf_utils
+from mistral_lib import actions as ml_actions
 
 
 # Use the set_default method to set value otherwise in certain test cases
@@ -32,13 +32,13 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
     def _run_workflow(self, wf_text, expected_state=states.ERROR):
         wf_service.create_workflows(wf_text)
 
-        wf_ex = self.engine.start_workflow('wf', {})
+        wf_ex = self.engine.start_workflow('wf', '', {})
 
         self.await_workflow_state(wf_ex.id, expected_state)
 
         return db_api.get_workflow_execution(wf_ex.id)
 
-    def test_direct_workflow_on_closures(self):
+    def test_on_closures(self):
         wf_text = """
         version: '2.0'
 
@@ -48,7 +48,7 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
           tasks:
             task1:
               description: |
-                Explicit 'fail' command should lead to workflow failure.
+                Explicit 'succeed' command should lead to workflow success.
               action: std.echo output="Echo"
               on-success:
                 - task2
@@ -72,23 +72,24 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
               action: std.noop
         """
 
-        wf_ex = self._run_workflow(wf_text)
+        wf_ex = self._run_workflow(wf_text, expected_state=states.SUCCESS)
 
-        tasks = wf_ex.task_executions
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            tasks = wf_ex.task_executions
 
         task1 = self._assert_single_item(tasks, name='task1')
-        task3 = self._assert_single_item(tasks, name='task3')
-        task4 = self._assert_single_item(tasks, name='task4')
+        task2 = self._assert_single_item(tasks, name='task2')
 
-        self.assertEqual(3, len(tasks))
+        self.assertEqual(2, len(tasks))
 
         self.await_task_success(task1.id)
-        self.await_task_success(task3.id)
-        self.await_task_success(task4.id)
+        self.await_task_success(task2.id)
 
         self.assertTrue(wf_ex.state, states.ERROR)
 
-    def test_direct_workflow_condition_transition_not_triggering(self):
+    def test_condition_transition_not_triggering(self):
         wf_text = """---
         version: '2.0'
 
@@ -112,12 +113,14 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
         """
 
         wf_service.create_workflows(wf_text)
-        wf_ex = self.engine.start_workflow('wf', {})
+        wf_ex = self.engine.start_workflow('wf', '', {})
 
         self.await_workflow_error(wf_ex.id)
 
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
-        tasks = wf_ex.task_executions
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            tasks = wf_ex.task_executions
 
         task1 = self._assert_single_item(tasks, name='task1')
 
@@ -127,7 +130,7 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
 
         self.assertTrue(wf_ex.state, states.ERROR)
 
-    def test_direct_workflow_change_state_after_success(self):
+    def test_change_state_after_success(self):
         wf_text = """
         version: '2.0'
 
@@ -144,7 +147,7 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
 
         wf_service.create_workflows(wf_text)
 
-        wf_ex = self.engine.start_workflow('wf', {})
+        wf_ex = self.engine.start_workflow('wf', '', {})
 
         self.await_workflow_success(wf_ex.id)
 
@@ -152,10 +155,12 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
             states.SUCCESS,
             self.engine.resume_workflow(wf_ex.id).state
         )
+
         self.assertRaises(
             exc.WorkflowException,
             self.engine.pause_workflow, wf_ex.id
         )
+
         self.assertEqual(
             states.SUCCESS,
             self.engine.stop_workflow(wf_ex.id, states.ERROR).state
@@ -240,7 +245,7 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
 
         wf_service.create_workflows(wf_text)
 
-        wf_ex = self.engine.start_workflow('wf', None)
+        wf_ex = self.engine.start_workflow('wf', '', None)
 
         self.assertIn(
             "Failed to find action [action_name=wrong.task]",
@@ -271,7 +276,10 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
         self.assertEqual(states.ERROR, wf_ex.state)
         self.assertIn('Can not evaluate YAQL expression', wf_ex.state_info)
 
-        task_execs = wf_ex.task_executions
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            task_execs = wf_ex.task_executions
 
         self.assertEqual(2, len(task_execs))
 
@@ -324,13 +332,15 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
         # and async action execution are RUNNING.
         wf_ex = self._run_workflow(wf_text, states.RUNNING)
 
-        self.assertEqual(states.RUNNING, wf_ex.state)
-        self.assertEqual(1, len(wf_ex.task_executions))
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
-        task_1_ex = self._assert_single_item(
-            wf_ex.task_executions,
-            name='task1'
-        )
+            task_execs = wf_ex.task_executions
+
+        self.assertEqual(states.RUNNING, wf_ex.state)
+        self.assertEqual(1, len(task_execs))
+
+        task_1_ex = self._assert_single_item(task_execs, name='task1')
 
         self.assertEqual(states.RUNNING, task_1_ex.state)
 
@@ -344,16 +354,17 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
         # Update async action execution result.
         self.engine.on_action_complete(
             task_1_action_exs[0].id,
-            wf_utils.Result(data='foobar')
+            ml_actions.Result(data='foobar')
         )
 
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            task_execs = wf_ex.task_executions
 
         self.assertEqual(states.ERROR, wf_ex.state)
         self.assertIn('Can not evaluate YAQL expression', wf_ex.state_info)
 
-        task_execs = wf_ex.task_executions
-
         self.assertEqual(2, len(task_execs))
 
         # 'task1' must be in SUCCESS.
@@ -397,10 +408,10 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
 
         wf_service.create_workflows(wf_text)
 
-        wf_ex = self.engine.start_workflow('wf', None)
+        wf_ex = self.engine.start_workflow('wf', '', None)
 
         self.assertIn(
-            "Can not evaluate YAQL expression:  wrong(yaql)",
+            "Can not evaluate YAQL expression [expression=wrong(yaql)",
             wf_ex.state_info
         )
         self.assertEqual(states.ERROR, wf_ex.state)
@@ -420,7 +431,7 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
 
         wf_service.create_workflows(wf_text)
 
-        wf_ex = self.engine.start_workflow('wf', {'var': 2})
+        wf_ex = self.engine.start_workflow('wf', '', {'var': 2})
 
         self.assertIn("Can not evaluate YAQL expression", wf_ex.state_info)
         self.assertEqual(states.ERROR, wf_ex.state)
@@ -451,7 +462,7 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
 
         wf_service.create_workflows(wf_text)
 
-        wf_ex = self.engine.start_workflow('wf', {})
+        wf_ex = self.engine.start_workflow('wf', '', {})
 
         self.await_workflow_success(wf_ex.id)
 
@@ -478,11 +489,16 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
         self.assertEqual(states.ERROR, wf_ex.state)
         self.assertIn('Can not evaluate YAQL expression', wf_ex.state_info)
 
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            task_execs = wf_ex.task_executions
+
         # Assert that there is only one task execution and it's SUCCESS.
-        self.assertEqual(1, len(wf_ex.task_executions))
+        self.assertEqual(1, len(task_execs))
 
         task_1_ex = self._assert_single_item(
-            wf_ex.task_executions,
+            task_execs,
             name='task1'
         )
 
@@ -518,12 +534,15 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
         wf_ex = self._run_workflow(wf_text, states.RUNNING)
 
         self.assertEqual(states.RUNNING, wf_ex.state)
-        self.assertEqual(1, len(wf_ex.task_executions))
 
-        task_1_ex = self._assert_single_item(
-            wf_ex.task_executions,
-            name='task1'
-        )
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            task_execs = wf_ex.task_executions
+
+        self.assertEqual(1, len(task_execs))
+
+        task_1_ex = self._assert_single_item(task_execs, name='task1')
 
         self.assertEqual(states.RUNNING, task_1_ex.state)
 
@@ -537,20 +556,20 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
         # Update async action execution result.
         self.engine.on_action_complete(
             task_1_action_exs[0].id,
-            wf_utils.Result(data='foobar')
+            ml_actions.Result(data='foobar')
         )
 
         # Assert that task1 is SUCCESS and workflow is ERROR.
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            task_execs = wf_ex.task_executions
 
         self.assertEqual(states.ERROR, wf_ex.state)
         self.assertIn('Can not evaluate YAQL expression', wf_ex.state_info)
-        self.assertEqual(1, len(wf_ex.task_executions))
+        self.assertEqual(1, len(task_execs))
 
-        task_1_ex = self._assert_single_item(
-            wf_ex.task_executions,
-            name='task1'
-        )
+        task_1_ex = self._assert_single_item(task_execs, name='task1')
 
         self.assertEqual(states.ERROR, task_1_ex.state)
 
@@ -581,7 +600,7 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
             wf_text
         )
 
-        self.assertIn("Task 'task3' not found", exception.message)
+        self.assertIn("Task 'task3' not found", str(exception))
 
     def test_delete_workflow_completion_check_on_stop(self):
         wf_text = """---
@@ -595,7 +614,7 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
 
         wf_service.create_workflows(wf_text)
 
-        wf_ex = self.engine.start_workflow('wf', {})
+        wf_ex = self.engine.start_workflow('wf', '', {})
 
         calls = db_api.get_delayed_calls()
 
@@ -622,7 +641,7 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
 
         wf_service.create_workflows(wf_text)
 
-        wf_ex = self.engine.start_workflow('wf', {})
+        wf_ex = self.engine.start_workflow('wf', '', {})
 
         calls = db_api.get_delayed_calls()
 
@@ -636,3 +655,102 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
             lambda:
             len(db_api.get_delayed_calls(target_method_name=mtd_name)) == 0
         )
+
+    def test_output(self):
+        wf_text = """---
+        version: '2.0'
+
+        wf:
+          tasks:
+            task1:
+              action: std.echo output="Hi Mistral!"
+              on-success: task2
+
+            task2:
+              action: std.noop
+        """
+
+        wf_service.create_workflows(wf_text)
+
+        wf_ex = self.engine.start_workflow('wf', '', {})
+
+        self.await_workflow_success(wf_ex.id)
+
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            self.assertDictEqual({}, wf_ex.output)
+
+    def test_triggered_by(self):
+        wf_text = """---
+        version: '2.0'
+
+        wf:
+          tasks:
+            task1:
+              action: std.noop
+              on-success: task2
+
+            task2:
+              action: std.fail
+              on-error: task3
+
+            task3:
+              action: std.fail
+              on-error: noop
+              on-success: task4
+              on-complete: task4
+
+            task4:
+              action: std.noop
+        """
+
+        wf_service.create_workflows(wf_text)
+
+        wf_ex = self.engine.start_workflow('wf', '', {})
+
+        self.await_workflow_success(wf_ex.id)
+
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            task_execs = wf_ex.task_executions
+
+        task1 = self._assert_single_item(task_execs, name='task1')
+        task2 = self._assert_single_item(task_execs, name='task2')
+        task3 = self._assert_single_item(task_execs, name='task3')
+        task4 = self._assert_single_item(task_execs, name='task4')
+
+        key = 'triggered_by'
+
+        self.assertIsNone(task1.runtime_context.get(key))
+
+        self.assertListEqual(
+            [
+                {
+                    "task_id": task1.id,
+                    "event": "on-success"
+                }
+            ],
+            task2.runtime_context.get(key)
+        )
+
+        self.assertListEqual(
+            [
+                {
+                    "task_id": task2.id,
+                    "event": "on-error"
+                }
+            ],
+            task3.runtime_context.get(key)
+        )
+
+        self.assertListEqual(
+            [
+                {
+                    "task_id": task3.id,
+                    "event": "on-complete"
+                }
+            ],
+            task4.runtime_context.get(key)
+        )
