@@ -21,10 +21,10 @@ from mistral.db.v2 import api as db_api
 from mistral.db.v2.sqlalchemy import models
 from mistral.engine import policies
 from mistral import exceptions as exc
+from mistral.lang import parser as spec_parser
 from mistral.services import workbooks as wb_service
 from mistral.services import workflows as wf_service
 from mistral.tests.unit.engine import base
-from mistral.workbook import parser as spec_parser
 from mistral.workflow import states
 
 
@@ -423,7 +423,7 @@ class PoliciesTest(base.EngineTestCase):
         wb_service.create_workbook_v2(WAIT_BEFORE_WB)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {})
+        wf_ex = self.engine.start_workflow('wb.wf1', '', {})
 
         with db_api.transaction():
             # Note: We need to reread execution to access related tasks.
@@ -443,7 +443,7 @@ class PoliciesTest(base.EngineTestCase):
         wb_service.create_workbook_v2(WAIT_BEFORE_FROM_VAR)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {'wait_before': 1})
+        wf_ex = self.engine.start_workflow('wb.wf1', '', {'wait_before': 1})
 
         with db_api.transaction():
             # Note: We need to reread execution to access related tasks.
@@ -471,7 +471,7 @@ class PoliciesTest(base.EngineTestCase):
 
         wf_service.create_workflows(wf_text)
 
-        wf_ex = self.engine.start_workflow('wf', {})
+        wf_ex = self.engine.start_workflow('wf', '', {})
 
         self.await_workflow_success(wf_ex.id)
 
@@ -488,7 +488,7 @@ class PoliciesTest(base.EngineTestCase):
         wb_service.create_workbook_v2(WAIT_AFTER_WB)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {})
+        wf_ex = self.engine.start_workflow('wb.wf1', '', {})
 
         with db_api.transaction():
             # Note: We need to reread execution to access related tasks.
@@ -506,7 +506,7 @@ class PoliciesTest(base.EngineTestCase):
         wb_service.create_workbook_v2(WAIT_AFTER_FROM_VAR)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {'wait_after': 2})
+        wf_ex = self.engine.start_workflow('wb.wf1', '', {'wait_after': 2})
 
         with db_api.transaction():
             # Note: We need to reread execution to access related tasks.
@@ -531,7 +531,7 @@ class PoliciesTest(base.EngineTestCase):
         wb_service.create_workbook_v2(RETRY_WB)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {})
+        wf_ex = self.engine.start_workflow('wb.wf1', '', {})
 
         with db_api.transaction():
             # Note: We need to reread execution to access related tasks.
@@ -561,7 +561,11 @@ class PoliciesTest(base.EngineTestCase):
         wb_service.create_workbook_v2(RETRY_WB_FROM_VAR)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {'count': 3, 'delay': 1})
+        wf_ex = self.engine.start_workflow(
+            'wb.wf1',
+            '',
+            {'count': 3, 'delay': 1}
+        )
 
         with db_api.transaction():
             # Note: We need to reread execution to access related tasks.
@@ -572,11 +576,20 @@ class PoliciesTest(base.EngineTestCase):
         self.assertEqual(states.RUNNING, task_ex.state)
         self.assertDictEqual({}, task_ex.runtime_context)
 
-        # TODO(rakhmerov): This check doesn't make sense anymore because
-        # we don't store evaluated values anywhere.
-        # Need to create a better test.
-        # self.assertEqual(3, task_ex.in_context["count"])
-        # self.assertEqual(1, task_ex.in_context["delay"])
+        self.await_task_delayed(task_ex.id, delay=0.5)
+        self.await_task_error(task_ex.id)
+
+        self.await_workflow_error(wf_ex.id)
+
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            task_ex = wf_ex.task_executions[0]
+
+        self.assertEqual(
+            3,
+            task_ex.runtime_context["retry_task_policy"]["retry_no"]
+        )
 
     def test_retry_policy_never_happen(self):
         retry_wb = """---
@@ -597,7 +610,7 @@ class PoliciesTest(base.EngineTestCase):
         wb_service.create_workbook_v2(retry_wb)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {})
+        wf_ex = self.engine.start_workflow('wb.wf1', '', {})
 
         with db_api.transaction():
             # Note: We need to reread execution to access related tasks.
@@ -640,7 +653,7 @@ class PoliciesTest(base.EngineTestCase):
         wb_service.create_workbook_v2(retry_wb)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {})
+        wf_ex = self.engine.start_workflow('wb.wf1', '', {})
 
         with db_api.transaction():
             # Note: We need to reread execution to access related tasks.
@@ -683,7 +696,7 @@ class PoliciesTest(base.EngineTestCase):
         wb_service.create_workbook_v2(retry_wb)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {})
+        wf_ex = self.engine.start_workflow('wb.wf1', '', {})
 
         with db_api.transaction():
             # Note: We need to reread execution to access related tasks.
@@ -727,7 +740,7 @@ class PoliciesTest(base.EngineTestCase):
         wb_service.create_workbook_v2(retry_wb)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {})
+        wf_ex = self.engine.start_workflow('wb.wf1', '', {})
 
         with db_api.transaction():
             # Note: We need to reread execution to access related tasks.
@@ -768,7 +781,7 @@ class PoliciesTest(base.EngineTestCase):
         wb_service.create_workbook_v2(retry_wb)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {})
+        wf_ex = self.engine.start_workflow('wb.wf1', '', {})
 
         with db_api.transaction():
             # Note: We need to reread execution to access related tasks.
@@ -807,7 +820,7 @@ class PoliciesTest(base.EngineTestCase):
         wb_service.create_workbook_v2(retry_wb)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {})
+        wf_ex = self.engine.start_workflow('wb.wf1', '', {})
 
         with db_api.transaction():
             # Note: We need to reread execution to access related tasks.
@@ -853,7 +866,7 @@ class PoliciesTest(base.EngineTestCase):
         wb_service.create_workbook_v2(retry_wb)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.main', {})
+        wf_ex = self.engine.start_workflow('wb.main', '', {})
 
         with db_api.transaction():
             # Note: We need to reread execution to access related tasks.
@@ -899,7 +912,7 @@ class PoliciesTest(base.EngineTestCase):
         wb_service.create_workbook_v2(retry_wb)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {})
+        wf_ex = self.engine.start_workflow('wb.wf1', '', {})
 
         with db_api.transaction():
             # Note: We need to reread execution to access related tasks.
@@ -952,7 +965,7 @@ class PoliciesTest(base.EngineTestCase):
         """
 
         wf_service.create_workflows(retry_wf)
-        wf_ex = self.engine.start_workflow('wf1', {})
+        wf_ex = self.engine.start_workflow('wf1', '', {})
 
         self.await_workflow_success(wf_ex.id)
 
@@ -975,7 +988,7 @@ class PoliciesTest(base.EngineTestCase):
         wb_service.create_workbook_v2(TIMEOUT_WB)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {})
+        wf_ex = self.engine.start_workflow('wb.wf1', '', {})
 
         with db_api.transaction():
             # Note: We need to reread execution to access related tasks.
@@ -1000,7 +1013,7 @@ class PoliciesTest(base.EngineTestCase):
         wb_service.create_workbook_v2(TIMEOUT_WB2)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {})
+        wf_ex = self.engine.start_workflow('wb.wf1', '', {})
 
         with db_api.transaction():
             # Note: We need to reread execution to access related tasks.
@@ -1027,7 +1040,7 @@ class PoliciesTest(base.EngineTestCase):
         wb_service.create_workbook_v2(TIMEOUT_FROM_VAR)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {'timeout': 1})
+        wf_ex = self.engine.start_workflow('wb.wf1', '', {'timeout': 1})
 
         with db_api.transaction():
             # Note: We need to reread execution to access related tasks.
@@ -1046,7 +1059,7 @@ class PoliciesTest(base.EngineTestCase):
         wb_service.create_workbook_v2(PAUSE_BEFORE_WB)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {})
+        wf_ex = self.engine.start_workflow('wb.wf1', '', {})
 
         with db_api.transaction():
             wf_ex = db_api.get_workflow_execution(wf_ex.id)
@@ -1087,7 +1100,7 @@ class PoliciesTest(base.EngineTestCase):
         wb_service.create_workbook_v2(PAUSE_BEFORE_DELAY_WB)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {})
+        wf_ex = self.engine.start_workflow('wb.wf1', '', {})
 
         with db_api.transaction():
             wf_ex = db_api.get_workflow_execution(wf_ex.id)
@@ -1139,7 +1152,7 @@ class PoliciesTest(base.EngineTestCase):
         wb_service.create_workbook_v2(CONCURRENCY_WB)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {})
+        wf_ex = self.engine.start_workflow('wb.wf1', '', {})
 
         self.await_workflow_success(wf_ex.id)
 
@@ -1157,7 +1170,7 @@ class PoliciesTest(base.EngineTestCase):
         wb_service.create_workbook_v2(CONCURRENCY_WB_FROM_VAR)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {'concurrency': 4})
+        wf_ex = self.engine.start_workflow('wb.wf1', '', {'concurrency': 4})
 
         with db_api.transaction():
             wf_ex = db_api.get_workflow_execution(wf_ex.id)
@@ -1185,7 +1198,7 @@ class PoliciesTest(base.EngineTestCase):
         wb_service.create_workbook_v2(wb)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {'wait_before': '1'})
+        wf_ex = self.engine.start_workflow('wb.wf1', '', {'wait_before': '1'})
 
         self.assertIn(
             'Invalid data type in WaitBeforePolicy',
@@ -1210,7 +1223,7 @@ class PoliciesTest(base.EngineTestCase):
         wf_service.create_workflows(wf_delayed_state)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wf', {})
+        wf_ex = self.engine.start_workflow('wf', '', {})
 
         self.await_workflow_success(wf_ex.id)
 
