@@ -21,7 +21,7 @@ from mistral.db.v2 import api as db_api
 from mistral.services import workbooks as wb_service
 from mistral.tests.unit.engine import base
 from mistral.workflow import states
-from mistral.workflow import utils as wf_utils
+from mistral_lib import actions as ml_actions
 
 
 # Use the set_default method to set value otherwise in certain test cases
@@ -63,7 +63,7 @@ class ReverseWorkflowRerunCancelledTest(base.EngineTestCase):
 
         wb_service.create_workbook_v2(wb_def)
 
-        wf1_ex = self.engine.start_workflow('wb1.wf1', {}, task_name='t3')
+        wf1_ex = self.engine.start_workflow('wb1.wf1', '', {}, task_name='t3')
 
         self.await_workflow_state(wf1_ex.id, states.RUNNING)
 
@@ -86,7 +86,7 @@ class ReverseWorkflowRerunCancelledTest(base.EngineTestCase):
         # Cancel action execution for task.
         self.engine.on_action_complete(
             wf1_t1_action_exs[0].id,
-            wf_utils.Result(cancel=True)
+            ml_actions.Result(cancel=True)
         )
 
         self.await_workflow_cancelled(wf1_ex.id)
@@ -142,7 +142,7 @@ class ReverseWorkflowRerunCancelledTest(base.EngineTestCase):
 
         self.engine.on_action_complete(
             wf1_t1_action_exs[1].id,
-            wf_utils.Result(data={'foo': 'bar'})
+            ml_actions.Result(data={'foo': 'bar'})
         )
 
         # Wait for the workflow to succeed.
