@@ -16,8 +16,8 @@
 import mock
 
 from mistral.db.v2 import api as db_api
-from mistral.engine import default_executor
-from mistral.engine.rpc_backend import rpc
+from mistral.executors import default_executor as d_exe
+from mistral.executors import remote_executor as r_exe
 from mistral.services import workflows as wf_service
 from mistral.tests.unit.engine import base
 from mistral.workflow import data_flow
@@ -25,9 +25,9 @@ from mistral.workflow import states
 
 
 def _run_at_target(action_ex_id, action_class_str, attributes,
-                   action_params, target=None, async_=True, safe_rerun=False):
+                   action_params, safe_rerun, target=None, async_=True):
     # We'll just call executor directly for testing purposes.
-    executor = default_executor.DefaultExecutor()
+    executor = d_exe.DefaultExecutor()
 
     executor.run_action(
         action_ex_id,
@@ -43,7 +43,8 @@ MOCK_RUN_AT_TARGET = mock.MagicMock(side_effect=_run_at_target)
 
 
 class TestSafeRerun(base.EngineTestCase):
-    @mock.patch.object(rpc.ExecutorClient, 'run_action', MOCK_RUN_AT_TARGET)
+
+    @mock.patch.object(r_exe.RemoteExecutor, 'run_action', MOCK_RUN_AT_TARGET)
     def test_safe_rerun_true(self):
         wf_text = """---
         version: '2.0'
@@ -71,7 +72,7 @@ class TestSafeRerun(base.EngineTestCase):
         # to true.
 
         wf_service.create_workflows(wf_text)
-        wf_ex = self.engine.start_workflow('wf', {})
+        wf_ex = self.engine.start_workflow('wf', '', {})
 
         self.await_workflow_success(wf_ex.id)
 
@@ -89,7 +90,7 @@ class TestSafeRerun(base.EngineTestCase):
         self.assertEqual(task1.state, states.SUCCESS)
         self.assertEqual(task2.state, states.SUCCESS)
 
-    @mock.patch.object(rpc.ExecutorClient, 'run_action', MOCK_RUN_AT_TARGET)
+    @mock.patch.object(r_exe.RemoteExecutor, 'run_action', MOCK_RUN_AT_TARGET)
     def test_safe_rerun_false(self):
         wf_text = """---
         version: '2.0'
@@ -117,7 +118,7 @@ class TestSafeRerun(base.EngineTestCase):
         # to true.
 
         wf_service.create_workflows(wf_text)
-        wf_ex = self.engine.start_workflow('wf', {})
+        wf_ex = self.engine.start_workflow('wf', '', {})
 
         self.await_workflow_success(wf_ex.id)
 
@@ -135,7 +136,7 @@ class TestSafeRerun(base.EngineTestCase):
         self.assertEqual(task1.state, states.ERROR)
         self.assertEqual(task3.state, states.SUCCESS)
 
-    @mock.patch.object(rpc.ExecutorClient, 'run_action', MOCK_RUN_AT_TARGET)
+    @mock.patch.object(r_exe.RemoteExecutor, 'run_action', MOCK_RUN_AT_TARGET)
     def test_safe_rerun_with_items(self):
         wf_text = """---
         version: '2.0'
@@ -152,7 +153,7 @@ class TestSafeRerun(base.EngineTestCase):
         """
 
         wf_service.create_workflows(wf_text)
-        wf_ex = self.engine.start_workflow('wf', {})
+        wf_ex = self.engine.start_workflow('wf', '', {})
 
         self.await_workflow_success(wf_ex.id)
 
