@@ -105,7 +105,7 @@ class SubworkflowsTest(base.EngineTestCase):
         wb_service.create_workbook_v2(WB2)
 
     def test_subworkflow_success(self):
-        wf2_ex = self.engine.start_workflow('wb1.wf2', None)
+        wf2_ex = self.engine.start_workflow('wb1.wf2', '', None)
 
         project_id = auth_context.ctx().project_id
 
@@ -113,7 +113,7 @@ class SubworkflowsTest(base.EngineTestCase):
         self.assertEqual(project_id, wf2_ex.project_id)
         self.assertIsNotNone(wf2_ex)
         self.assertDictEqual({}, wf2_ex.input)
-        self.assertDictEqual({}, wf2_ex.params)
+        self.assertDictEqual({'namespace': ''}, wf2_ex.params)
 
         self._await(lambda: len(db_api.get_workflow_executions()) == 2, 0.5, 5)
 
@@ -187,7 +187,7 @@ class SubworkflowsTest(base.EngineTestCase):
     @mock.patch.object(std_actions.EchoAction, 'run',
                        mock.MagicMock(side_effect=exc.ActionException))
     def test_subworkflow_error(self):
-        self.engine.start_workflow('wb1.wf2', None)
+        self.engine.start_workflow('wb1.wf2', '', None)
 
         self._await(lambda: len(db_api.get_workflow_executions()) == 2, 0.5, 5)
 
@@ -205,7 +205,7 @@ class SubworkflowsTest(base.EngineTestCase):
         self.await_workflow_error(wf2_ex.id)
 
     def test_subworkflow_yaql_error(self):
-        wf_ex = self.engine.start_workflow('wb2.wf1', None)
+        wf_ex = self.engine.start_workflow('wb2.wf1', '', None)
 
         self.await_workflow_error(wf_ex.id)
 
@@ -227,12 +227,15 @@ class SubworkflowsTest(base.EngineTestCase):
     def test_subworkflow_environment_inheritance(self):
         env = {'key1': 'abc'}
 
-        wf2_ex = self.engine.start_workflow('wb1.wf2', None, env=env)
+        wf2_ex = self.engine.start_workflow('wb1.wf2', '', None, env=env)
 
         # Execution of 'wf2'.
         self.assertIsNotNone(wf2_ex)
         self.assertDictEqual({}, wf2_ex.input)
-        self.assertDictEqual({'env': env}, wf2_ex.params)
+        self.assertDictEqual(
+            {'env': env, 'namespace': ''},
+            wf2_ex.params
+        )
 
         self._await(lambda: len(db_api.get_workflow_executions()) == 2, 0.5, 5)
 
