@@ -22,9 +22,9 @@ from mistral import context as auth_ctx
 from mistral.db.v2.sqlalchemy import models
 from mistral import exceptions as exc
 from mistral import expressions as expr
+from mistral.lang import parser as spec_parser
 from mistral import utils
 from mistral.utils import inspect_utils
-from mistral.workbook import parser as spec_parser
 from mistral.workflow import states
 
 LOG = logging.getLogger(__name__)
@@ -190,25 +190,38 @@ def publish_variables(task_ex, task_spec):
 
     wf_ex = task_ex.workflow_execution
 
-    expr_ctx = ContextView(
-        task_ex.in_context,
-        wf_ex.context,
-        wf_ex.input
-    )
+    expr_ctx = ContextView(task_ex.in_context, wf_ex.context, wf_ex.input)
 
     if task_ex.name in expr_ctx:
         LOG.warning(
-            'Shadowing context variable with task name while publishing: %s' %
+            'Shadowing context variable with task name while '
+            'publishing: %s',
             task_ex.name
         )
 
-    data = (
-        task_spec.get_publish()
-        if task_ex.state == states.SUCCESS
-        else task_spec.get_publish_on_error()
+    publish_spec = task_spec.get_publish(task_ex.state)
+
+    if not publish_spec:
+        return
+
+    # Publish branch variables.
+    branch_vars = publish_spec.get_branch()
+
+    task_ex.published = expr.evaluate_recursively(branch_vars, expr_ctx)
+
+    # Publish global variables.
+    global_vars = publish_spec.get_global()
+
+    utils.merge_dicts(
+        task_ex.workflow_execution.context,
+        expr.evaluate_recursively(global_vars, expr_ctx)
     )
 
-    task_ex.published = expr.evaluate_recursively(data, expr_ctx)
+    # TODO(rakhmerov):
+    # 1. Publish atomic variables.
+    # 2. Add the field "publish" in TaskExecution model similar to "published"
+    #    but containing info as
+    #    {'branch': {vars}, 'global': {vars}, 'atomic': {vars}}
 
 
 def evaluate_task_outbound_context(task_ex):
@@ -265,8 +278,6 @@ def add_openstack_data_to_context(wf_ex):
     if CONF.pecan.auth_enable:
         exec_ctx = auth_ctx.ctx()
 
-        LOG.debug('Data flow security context: %s' % exec_ctx)
-
         if exec_ctx:
             wf_ex.context.update({'openstack': exec_ctx.to_dict()})
 
